SimpleActionSets = LibStub("AceAddon-3.0"):NewAddon("SimpleActionSets", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub( "AceLocale-3.0" ):GetLocale( "SimpleActionSets" )

local SAS_Temp;
local SAS_SavedPickup;
local SAS_ReturnAction;
local SAS_DraggingBar;
local PlrName;
local inCombat;
local delayedSet;
local delayedSetPlayer;
local SAS_SwappingSet;

-- Basic data broker support
local ldb = LibStub("LibDataBroker-1.1"):NewDataObject("SimpleActionSets", {
	type = "launcher",
	label = "SimpleActionSets",
	icon = "Interface\\AddOns\\SimpleActionSets\\sas",
	OnClick = function(self,button)
		if button == "RightButton" then
			ToggleDropDownMenu(1, nil, SAS_SetsDropDown, self, 0, 0);
		else
			SimpleActionSets:Main_Toggle()
		end
	end,
	OnTooltipShow = function(tt)
		tt:AddLine("Simple Action Sets")
		tt:AddDoubleLine("Current Set", SimpleActionSets:GetCurrentSet(), 1,1,1, 0.75,0.75,0.75)
	end,
})

local defaults = {
	profile = {
		miniMapButton = {
			hide = false,
			lock = false,
			position = 270,
			offset = 0
		},
		noUIWarnings = false,
		emptyBars = false,
		noEmptyButtons = false,
		autoRestore = false,
		noAutoRestoreWarning = false,
		noRemove = false,
		currentSet = nil,
		sets = {},
	}
}

function SimpleActionSets:OnInitialize()
    --Called when the addon is loaded
    self.db = LibStub("AceDB-3.0"):New("SAS_Saved", defaults)
	self:RegisterChatCommand("SAS", "Console")
end

function SimpleActionSets:OnEnable()
    --Called when the addon is enabled
	self:RegisterEvent("ACTIONBAR_SHOWGRID");
	self:RegisterEvent("ACTIONBAR_HIDEGRID");
	self:RegisterEvent("UNIT_INVENTORY_CHANGED");
	self:RegisterEvent("BAG_UPDATE");
	self:RegisterEvent("PLAYER_LOGOUT");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")

	self:Minimap_PosUpdate();

	PlrName = UnitName("player").." - "..GetCVar("realmName");
	
	if ( self.db.profile.miniMapButton.hide ) then
		SASMinimapFrame:Hide();
	end
	
	local currentset = self:GetCurrentSet();
	local liveactions = self:IterateActions();

	if ( self.db.profile.autoRestore and self.db.profiles["BackUp"] and self.db.profiles["BackUp"].sets[PlrName] ) then
		if ( self:CompareSet( liveactions, self.db.profiles["BackUp"].sets[PlrName] ) ) then
			self:Warning( "CHANGEDSINCELAST", self.RestoreBackUp, nil, 1 );
			return;
		end
	end
	if ( currentset ) then
		if ( self:CompareSet( liveactions, self.db.profile.sets[currentset] ) ) then
			self:Debug( currentset.." does not appear to be loaded." );
			self:SetCurrentSet();
		else
			self:Debug( currentset.." appears to be loaded, keeping as current set." );
		end
	end
end

function SimpleActionSets:OnDisable()
    --Called when the addon is enabled
end

------------------------------
--      Event Functions     --
------------------------------

function SimpleActionSets:ACTIONBAR_SHOWGRID()
	if ( not SAS_SwappingSet ) then
		SAS_SavedPickup = self:BuildCursorAction();
		--self:Debug("Valid Item on Cursor");
	end
end

function SimpleActionSets:ACTIONBAR_HIDEGRID()
	if ( not SAS_SwappingSet ) then
		SAS_SavedPickup = nil;
		SAS_ReturnAction = nil;
		--self:Debug("No Valid Item on Cursor");
	end
end

function SimpleActionSets:UNIT_INVENTORY_CHANGED(arg1)
	self:CheckForMissingItems( arg1 )
end

function SimpleActionSets:BAG_UPDATE(arg1)
	self:CheckForMissingItems( arg1 )
end

function SimpleActionSets:CheckForMissingItems( arg1 )
	if ( arg1 == "player" or tonumber(arg1) ) then
		self:FindMissingItems();
		self:Actions_Display();
	end
end

function SimpleActionSets:PLAYER_LOGOUT()
	if ( not self.db.profiles["BackUp"] ) then
		self.db.profiles["BackUp"] = {};
		self.db.profiles["BackUp"].sets = {};
	end
	self.db.profiles["BackUp"].sets[PlrName] = self:IterateActions();
end

function SimpleActionSets:PLAYER_REGEN_DISABLED()
	inCombat = 1;
end

function SimpleActionSets:PLAYER_REGEN_ENABLED()
	inCombat = nil;
end

function SimpleActionSets:PLAYER_SPECIALIZATION_CHANGED()
	if (SASMain:IsShown()) then
		self:Actions_Display();
	end
end

----------------------------
--    Get Set Methods     --
----------------------------

function SimpleActionSets:GetPlrName()
	return PlrName;
end

function SimpleActionSets:GetTemp(x, y)
	if (x and y) then
		return SAS_Temp[x][y];
	elseif (x) then
		return SAS_Temp[x];
	else
		return SAS_Temp;
	end
end

function SimpleActionSets:SetTemp(x, y, value)
	if (x and y) then
		SAS_Temp[x][y] = value;
	elseif (x) then
		SAS_Temp[x] = value;
	else
		SAS_Temp = value
	end
end

function SimpleActionSets:GetSavedPickup()
	return SAS_SavedPickup;
end

function SimpleActionSets:GetReturnAction()
	return SAS_ReturnAction;
end

function SimpleActionSets:GetCombatStatus()
	return inCombat;
end

function SimpleActionSets:GetDraggingBar()
	return SAS_DraggingBar;
end

function SimpleActionSets:SetDraggingBar(value)
	SAS_DraggingBar = value;
end

------------------------------
-- SAS Main Frame functions --
------------------------------

local SASMainTabs = {
	{280,"SASSets"},
	{185,"SASOptions"},
	};

function SimpleActionSets:Main_UpdateDisplay()
	SASSets:Hide()
	SASOptions:Hide();
	local tab = SASMain.selectedTab;
	local height = SASMainTabs[tab][1];
	SASActions:SetHeight(height);
	SASMainBottomDivider:SetPoint("TOPLEFT", SASMainTopDivider, "BOTTOMLEFT", 0, 10-height);
	_G[SASMainTabs[tab][2]]:Show();
end

function SimpleActionSets:Main_Toggle()
	if ( SASMain:IsVisible() ) then
		HideUIPanel(SASMain);
	else
		ShowUIPanel(SASMain);
	end
end

-----------------------------
-- Actions Frame functions --
-----------------------------

function SimpleActionSets:BarEnabled( bar, set )
	if ( not set ) then set = SAS_Temp; end
	if ( set and set[bar] ) then
		return not set[bar][0];
	end
end

--------------------------------------
-- Warning and Save Frame Functions --
--------------------------------------
function SimpleActionSets:Warning( type, func, value, force )
	if ( self.db.profile.noUIWarnings and ( not force or self.db.profile.noAutoRestoreWarnings ) ) then
		func(value);
	else
		if ( value ) then
			SASWarningFrameText:SetText( gsub(L["SAS_TEXT_WARNING_"..type], "%%s", value) );
		else
			SASWarningFrameText:SetText( L["SAS_TEXT_WARNING_"..type] );
		end
		SASWarningFrame.func = func;
		SASWarningFrame.value = value
		ShowUIPanel(SASWarningFrame);
	end
end

function SimpleActionSets:Warning_Okay( self )
	SASWarningFrame.func(SASWarningFrame.value)
	SASWarningFrame.func = nil;
	SASWarningFrame.value = nil;
	HideUIPanel(self:GetParent());
end

function SimpleActionSets:SaveMenu_Save( set )
	SASSaveMenuHelp.warned = nil
	SASSaveMenuHelp:Hide();
	self:SaveSet( set );
	HideUIPanel( SASSaveMenu );
end


-------------------------
-- Fake Drag functions --
-------------------------
function SimpleActionSets:FakeDrag_Drop(clear)
	-- Pick up sas item
	SASFakeDragFrame:Hide();
	local focus = GetMouseFocus();
	local action = SASFakeDragFrame.Action;
	local bar = SASFakeDragFrame.Bar;
	if ( focus and not (focus.IsSASAction or focus.IsSASBar) or clear ) then
		if ( SASFakeDragFrame.Action ) then
			PlaySoundFile("Sound\\Interface\\uSpellIconDrop.wav");
			self:Debug("Dumping FakeDrag Action "..action);
		elseif ( SASFakeDragFrame.Bar ) then
			PlaySoundFile("Sound\\Interface\\uSpellIconDrop.wav");
			self:Debug("Dumping FakeDrag Bar");
		else
			self:Debug("Dumping FakeDrag Action, no action");
		end
		SASFakeDragFrame.Action = nil;
		SASFakeDragFrame.Bar = nil;
		SASFakeDragFrame:SetScript("OnUpdate", nil);
		if ( SAS_DraggingBar ) then
			if ( SASActions:IsVisible() ) then
				self:Actions_UpdateBar(SAS_DraggingBar);
			else
				SAS_DraggingBar = nil;
			end
		end
	end
	return action or bar;
end

function SimpleActionSets:FakeDrag_PickupAction(action)
	SASFakeDragFrame.Action = action;
	SASFakeDragFrame.Bar = nil;
	if ( action ) then
		local name, texture = self:ParseActionInfo( action );
		for i=1, 12 do
			_G["SASFakeDragFrameIcon"..i]:SetTexture();
		end
		self:Debug("FakeDrag Pickup Action "..name);
		SASFakeDragFrame:Show();
		SASFakeDragFrameIcon:SetTexture( texture );
		SASFakeDragFrameIcon:SetAlpha( 0.5 );
		PlaySoundFile("Sound\\Interface\\uSpellIconPickup.wav");
		SASFakeDragFrame:SetScript("OnUpdate", FakeDrag_OnUpdate);
	else
		SASFakeDragFrame:Hide();
		SASFakeDragFrame:SetScript("OnUpdate", nil);
	end
end

function SimpleActionSets:FakeDrag_PickupBar(bar)
	SASFakeDragFrame.Bar = self:CopyTable(bar);
	SASFakeDragFrame.Action = nil;
	if ( bar ) then
		SASFakeDragFrameIcon:SetTexture();
		for i=1, 12 do
			local icon = _G["SASFakeDragFrameIcon"..i];
			if ( bar[i] ) then
				local texture = self:ParseActionInfo( bar[i], 2 );
				icon:SetTexture( texture );
				icon:SetTexCoord( 0, 1, 0, 1 );
			else
				icon:SetTexture("Interface\\Buttons\\UI-Quickslot");
				icon:SetTexCoord( 0.1875, 0.8125, 0.1875, 0.8125 );
			end
			icon:SetAlpha( 0.5 );
			SASFakeDragFrame:Show();
			SASFakeDragFrame:SetScript("OnUpdate", FakeDrag_OnUpdate);
		end
		PlaySoundFile("Sound\\Interface\\uSpellIconPickup.wav");
	else
		SASFakeDragFrame:Hide();
		SASFakeDragFrame:SetScript("OnUpdate", nil);
	end
end

function SimpleActionSets:FakeDrag_OnUpdate( self )
	-- Update the position of the FakeDrag frame to under the cursor
	if ( (self.Action or self.Bar) and self:IsVisible() ) then
		if ( SASActions:IsVisible() ) then
			local curX, curY = GetCursorPosition();
			local scale = UIParent:GetScale();
			self:SetPoint("CENTER", "UIParent", "BOTTOMLEFT", curX/scale, curY/scale );
		else
			SimpleActionSets:FakeDrag_Drop(1);
			self:SetScript("OnUpdate", nil);
		end
	else
		self:SetScript("OnUpdate", nil);
	end
end

------------------------
-- Reusable functions --
------------------------
function SimpleActionSets:Debug(msg)
	-- Print when debug is turned on
	if ( self.db["debug"] ) then self:Print("<|wDEBUG|r> "..msg); end
end

function SimpleActionSets:TooltipAddLine(msg)
	-- Adds a colored line to the tooltip
	GameTooltip:AddLine( msg, 0.012, 0.658, 0.62, 1, 1 );
	GameTooltip:Show();
end

function SimpleActionSets:BuildActionInfo(...) -- name, texture, rank, link, macro, ?
	local temp, string, i = "";
	for i=1, select("#",...) do
		local val = select(i,...);
		if ( val ) then
			string = (string or "")..temp..(val or "").."";
			temp = "";
		else
			temp = temp .. "";
		end
	end
	return string;
end

function SimpleActionSets:IncActionInfo( action, val, part )
	if ( not val ) then return;
	elseif ( not part ) then part = 1; end
	local a = {self:ParseActionInfo(action)};
	a[part] = val;

	return self:BuildActionInfo( a[1], a[2], a[3], a[4], a[5], a[6] );
end

function SimpleActionSets:ParseActionInfo( action, ... )
	if not action then return; end
	if type(action)~="string" then return action; end
	local a = {strsplit("",action)};
	for k, v in pairs(a) do if v == "" then a[k] = nil; end end
	if ( select("#",...) > 0 ) then
		local b = {};
		for i=1, select("#",...) do
			tinsert( b, a[select(i,...)] );
		end
		return unpack(b);
	else
		return a[1], a[2], a[3], a[4], tonumber(a[5]); --name, texture, rank, link, macro, ?;
	end
end

function SimpleActionSets:FindLink( item )
	-- Find an item's link number from it's item link
	return strmatch( item, "item:(%d+):" );
end

function SimpleActionSets:FindName( item )
	-- Find an item's name from it's item link
	if ( item ) then
		for name in gmatch( item, "%[(.+)%]") do
			return name;
		end
	end
end

function SimpleActionSets:FindItem( item )
	-- Iterate over items the player has and return it's link and location --
	if ( not item ) then return; end

	item = tostring(item);

	-- Iterate over bags
	for i=0, 4 do
		local bagSlots = GetContainerNumSlots(i);
		if ( bagSlots ) then
			for j=1, bagSlots do
				self:Debug("Doing GetContainerItemLink() on "..i.." "..j);
				local itemLink = GetContainerItemLink(i,j);
				if ( itemLink ) then
					if ( item == self:FindLink( itemLink ) or item == self:FindName( itemLink ) ) then
						return itemLink, i, j;
					end
				end
			end
		end
	end

	-- Iterate over paper doll
	for i=0, 23 do
		self:Debug("Doing GetInventoryItemLink() on "..i);
		local itemLink = GetInventoryItemLink("player",i);
		if ( itemLink ) then
			if ( itemLink ) then
				if ( item == self:FindLink( itemLink ) or item == self:FindName( itemLink ) ) then
					return itemLink, nil, nil, i;
				end
			end
		end
	end
end

function SimpleActionSets:FindSpell( spell, rank )
	if ( not spell ) then return; end

	-- Iterate over spells the player has and return location
	local i = 1;

	for i = 1, self:GetNumSpellBookItems() do
		local spellName, spellRank =  GetSpellBookItemName( i, BOOKTYPE_SPELL );
		if (string.find(spell, "Call Pet ") and string.find(spellName, "Call ") and spell~=spellName) then
			--Catcher for Call Pet, stored spell is "Call Pet x", spellbook ones are "Call [PetName]"
			--If no pet in the slot this bit won't run as it'll be caught by following if statement.
			local _, _, petNnumber = strfind(spell,"(%d)")
			local _, petName = GetStablePetInfo(petNnumber)
			if (petName and spellName == "Call "..petName) then
				return i;
			end
		elseif ( spellName == spell ) then
			if ((spellRank == rank and rank) or (not rank)) then
				local _, spellId = GetSpellBookItemInfo(i, "spell");
				if ( IsSpellKnown(spellId) ) then
					--Spell is useable within current specialization
					return i;
				else
					return nil;
				end
			end
		end
		i = i+1;
	end
	
	return nil;
end

function SimpleActionSets:GetNumSpellBookItems()
	local t = GetNumSpellTabs();
	local n;
	while true do
		local name, _, offset, numSpells = GetSpellTabInfo(t);
		if not name then
			break
		end
		n = offset + numSpells;
		t = t + 1;
	end
	return n;
end

-- saved as globals in the macro blizz addon
-- MAX_ACCOUNT_MACROS = 36;
-- MAX_CHARACTER_MACROS = 18;

function SimpleActionSets:FindMacro( name, texture, macro )
	-- Check saved macro id
	local macroName, macroTexture, bestguess;
	if ( macro ) then
		macroName, macroTexture = GetMacroInfo(macro);
		if ( macroName == name ) then
			if ( texture and macroTexture == texture ) then
				return macro;
			end
			bestguess = macro;
		end
	end

	-- If no direct match, iterate over macros and return matching index
	local numAccountMacros, numCharacterMacros = GetNumMacros();
	numCharacterMacros = numCharacterMacros + 36;
	for i=1, 36+18 do
		if ( i > numAccountMacros and i < 37 ) then
			-- no more global macros, skip to local
			i = 37;
		end
		if ( i > numCharacterMacros ) then
			-- no more macros
			break;
		end
		macroName, macroTexture = GetMacroInfo(i)
		if ( macroName and macroName == name ) then
			if ( texture and macroTexture == texture ) then
				return i;
			end
			bestguess = i;
		end
	end

	return bestguess;
end

function SimpleActionSets:FindCompanion( name, companionType )
	local i;
	for i=1, GetNumCompanions(companionType) do
		local creatureID, creatureName, spellID, icon, _, mountFlags = GetCompanionInfo(companionType, i);
		if (name == creatureName or gsub(name,"Summon ","") == creatureName) then
			-- need to return the index
			return i, spellID;
		end
	end
end

function SimpleActionSets:GetActionInfo( id )
	local actionType, typeId, subType = GetActionInfo(id);

	if ( actionType == "spell" and typeId > 0 ) then
		local name, rank, texture = GetSpellInfo(typeId);
		return name, texture, rank;
	elseif ( actionType == "item" ) then
		local texture = GetActionTexture(id);
		return (GetItemInfo(typeId)), texture, nil, typeId;
	elseif ( actionType == "macro" and typeId > 0) then
		local name, texture = GetMacroInfo(typeId);
		if ( not texture ) then
			texture = GetActionTexture(id);
		end
		return name, texture, nil, nil, typeId;
	elseif ( actionType == "companion" ) then
		local name = GetSpellInfo(typeId)
		local companionId = self:FindCompanion( name, subType )
		local _, _, _, texture = GetCompanionInfo( subType, companionId );
		return name, texture, subType; -- treat companions as spells for now
	elseif ( actionType == "summonpet") then
		local _, _, _, _, _, _, name, texture = C_PetJournal.GetPetInfoByPetID( typeId );
		--typeID appears to be unique identifier for battle pets
		return name, texture, actionType, typeId;
	elseif ( actionType == "equipmentset" ) then
		local icon = GetEquipmentSetInfoByName(typeId);
		return typeId, "Interface\\Icons\\"..icon, actionType;
	elseif ( actionType == "flyout" ) then
		--GetCursorInfo seems to be only way of finding a flyout's texture
		self:PickupFlyout(typeId);
		local _, _, texture = GetCursorInfo();
		ClearCursor();
		local name = GetFlyoutInfo(typeId);
		return name, texture, actionType, typeId;
	end
end

function SimpleActionSets:IterateActions()
	-- Iterate over all current actions
	local actionlist = {};
	for i=0, 9 do
		actionlist[i] = {};
		for j=1, 12 do
			actionlist[i][j] = self:BuildActionInfo( self:GetActionInfo( j+(i*12) ) ); --action[1], action[2], action[3], action[4], action[5] );
		end
	end

	return actionlist;
end

function SimpleActionSets:SetTooltip( self, bar, id )
	-- Sets the Tooltip for the found action
	if ( not bar or not id ) then return; end

	if ( SAS_Temp[bar][id] ) then
		if ( SAS_Temp[bar][id] == 0 ) then
			GameTooltip:SetText( L["SAS_TEXT_TOOLTIP_FORCECLEAR"], 1, 1, 1 );
			SimpleActionSets:TooltipAddLine( L["SAS_TEXT_TOOLTIP_FORCECLEAR_TIP"] );
			GameTooltip:Show();
		elseif ( SAS_Temp[bar][id] == 1 ) then
			GameTooltip:SetText( L["SAS_TEXT_TOOLTIP_FORCENOAPPLY"], 1, 1, 1 );
			SimpleActionSets:TooltipAddLine( L["SAS_TEXT_TOOLTIP_FORCENOAPPLY_TIP"] );
			GameTooltip:Show();
		else
			local name, texture, rank, link, macro = SimpleActionSets:ParseActionInfo( SAS_Temp[bar][id] );
			local TooltipReturn;
			-- is a macro
			if ( macro ) then
				TooltipReturn = GameTooltip:SetText( name, 1, 1, 1 );
				local macroName,macroTexture,macroText = GetMacroInfo( SimpleActionSets:FindMacro(name, texture, macro) );
				if ( macroText ) then
					GameTooltip:AddLine( macroText, 0.75, 0.75, 0.75, 1, 1 );
					GameTooltip:Show();
				else
					SimpleActionSets:TooltipAddLine( L["SAS_TEXT_TOOLTIP_NOMACRO"] );
				end
			-- is an item
			elseif ( link and rank~="summonpet" and rank~="flyout") then
				local itemLink;
				SimpleActionSets:Debug("self:SetTooltip - link = "..link);
				if ( link == "?" or link == "1" ) then
					itemLink = SimpleActionSets:FindItem( name );
					if ( itemLink ) then
						link = SimpleActionSets:FindLink(itemLink);
					else
						link = nil;
					end
				else
					itemLink = SimpleActionSets:FindItem( link );
				end
				if ( link and select(2,GetItemInfo(link)) ) then
					TooltipReturn = GameTooltip:SetHyperlink(select(2,GetItemInfo(link)));
					if ( not itemLink ) then
						SimpleActionSets:TooltipAddLine( L["SAS_TEXT_TOOLTIP_NOTHAVE"] );
					end
				else
					TooltipReturn = GameTooltip:SetText( name, 1, 1, 1 );
					SimpleActionSets:TooltipAddLine( L["SAS_TEXT_TOOLTIP_NOTVALID"] );
				end
			-- is a companion
			elseif ( rank == "MOUNT" ) then
				local companionid, spellid = SimpleActionSets:FindCompanion( name, rank );
				if ( companionid ) then
					TooltipReturn = GameTooltip:SetHyperlink("spell:"..spellid);
				else
					TooltipReturn = GameTooltip:SetText( name, 1, 1, 1 );
					SimpleActionSets:TooltipAddLine( L["SAS_TEXT_TOOLTIP_NOCOMPANION"] );
				end
			-- is a battle pet
			elseif ( rank == "summonpet" ) then
				local _, _, _, _, _, _, battlePetName = C_PetJournal.GetPetInfoByPetID(link);
				TooltipReturn = GameTooltip:SetText( name, 1, 1, 1 );
				if ( battlePetName ) then
					GameTooltip:AddLine( L["SAS_PET_COMPANION_TOOLTIP_Line_1"], 1, 1, 1 );
					GameTooltip:AddLine( L["SAS_PET_COMPANION_TOOLTIP_Line_2"].."\n"..name.."." );
					GameTooltip:Show();
				else
					SimpleActionSets:TooltipAddLine( L["SAS_TEXT_TOOLTIP_NOBATTLEPET"] );
				end
			-- is an equipment set
			elseif ( rank == "equipmentset" ) then
				local equipSetIcon = GetEquipmentSetInfoByName(name);
				if ( equipSetIcon ) then
					TooltipReturn = GameTooltip:SetEquipmentSet(name);
				else
					TooltipReturn = GameTooltip:SetText( name, 1, 1, 1 );
					SimpleActionSets:TooltipAddLine( L["SAS_TEXT_TOOLTIP_NOEQUIPMENTSET"] );
				end
			-- is a flyout menu
			elseif ( rank == "flyout" ) then
				local _, description, _, isKnown = GetFlyoutInfo(link);
				TooltipReturn = GameTooltip:SetText( name, 1, 1, 1 );
				if (isKnown) then
					GameTooltip:AddLine( description );
					GameTooltip:Show();
				else
					SimpleActionSets:TooltipAddLine( L["SAS_TEXT_TOOLTIP_NOFLYOUT"] );
				end
			-- is a spell
			else
				local spellNum = SimpleActionSets:FindSpell( name, rank );
				if ( spellNum ) then
					TooltipReturn = GameTooltip:SetSpellBookItem( spellNum, BOOKTYPE_SPELL );
				else
					TooltipReturn = GameTooltip:SetText( name, 1, 1, 1 );
					SimpleActionSets:TooltipAddLine( L["SAS_TEXT_TOOLTIP_NOSPELL"] );
				end
			end
		end
		if ( TooltipReturn ) then
			self.updateTooltip = TOOLTIP_UPDATE_TIME;
		else
			self.updateTooltip = nil;
		end
	end
end

function SimpleActionSets:PickupFlyout(id)
	local i = 1;
	for i = 1, self:GetNumSpellBookItems() do
		local sbtype, sbid = GetSpellBookItemInfo(i, "spell")
		if (sbtype=="FLYOUT" and sbid==tonumber(id)) then
			PickupSpellBookItem(i, "spell")
			break;
		end
	end
end

function SimpleActionSets:BarHasActions(bar, set)
	if ( not set ) then
		set = SAS_Temp;
	end
	if ( bar and set[bar] ) then
		for i=1, 12 do
			if ( set[bar][i] ) then
				return 1;
			end
		end
	end
end

function SimpleActionSets:ClearSlot(id)
	-- Clear an action slot
	if ( HasAction(id) and not self.db.profile.noEmptyButtons ) then
		PickupAction(id);
		ClearCursor();
	end
end

function SimpleActionSets:MissingItem( id, itemInfo )
	-- Add to list of items to place on the bar when available.
	if ( id and itemInfo ) then
		if ( not self.db.profile["MissingItems"] ) then
			self.db.profile["MissingItems"] = {};
		end
		local missing = self.db.profile["MissingItems"]
		if ( not missing[id] or self:ParseActionInfo(missing[id], 3) ~= itemInfo[3] ) then
			self.db.profile["MissingItems"][id] = self:BuildActionInfo(itemInfo[1], itemInfo[2], nil, itemInfo[3], nil, nil, itemInfo[4]);
			if ( HasAction( id ) ) then
				PickupAction( id );
				ClearCursor();
			end
			self:Debug( itemInfo[1] );
		end
	end
end

function SimpleActionSets:FindMissingItems()
	-- Readd missing items to action bar
	if ( not self.db.profile["MissingItems"] ) then return; end
	for id, itemInfo in pairs(self.db.profile["MissingItems"]) do
		self:PlaceItem( id, self:ParseActionInfo( itemInfo, 4, 1, 6 ) );
	end
end

function SimpleActionSets:GetMissingItemInfo( id )
	if ( id ) then
		if ( self.db and self.db.profile ) then
			if ( self.db.profile["MissingItems"] and self.db.profile["MissingItems"][id] ) then
				return self.db.profile["MissingItems"][id];
			end
		end
	end
end

function SimpleActionSets:CheckItemDBs( name )
	-- check if there are any item db mods, get item link from that
	if ( ItemLinks ) then
		if ( ItemLinks[name] and ItemLinks[name]["i"] ) then
			self:Debug( "found "..name.." in LootLink db" );
			return self:tLink(ItemLinks[name]["i"]);
		end
	elseif ( IMDB ) then
		local realm = GetCVar("realmName");
		if ( IMDB[realm] and IMDB[realm][name] ) then
			for itemLink in gmatch( IMDB[realm][name], "item(%d:%d:%d:%d)" ) do
				self:Debug( "found "..name.." in ItemSync db" );
				return self:tLink(itemLink);
			end
		elseif ( IMDB[name] ) then
			for itemLink in gmatch( IMDB[name], "item(%d:%d:%d:%d)" ) do
				self:Debug( "found "..name.." in ItemSync db" );
				return self:tLink(itemLink);
			end
		end
	end

	self:Debug( "could not find "..name.." in either item dbs." );

	return "?";
end

function SimpleActionSets:PlaceItem( id, link, name, set )
	-- Place an item, update set link id if found
	local itemLink, bag, slot, inv;
	if ( link == "?" or link == "1" ) then
		itemLink, bag, slot, inv = self:CheckItem( name, id, set );
	else
		itemLink, bag, slot, inv = self:FindItem( link );
	end
	if ( bag ) then
		PickupContainerItem( bag, slot );
		PlaceAction(id);
		return 1;
	elseif ( inv ) then
		PickupInventoryItem( inv );
		PlaceAction(id);
		return 1;
	end
end

function SimpleActionSets:CheckItem( name, id, set )
	if ( name ) then
		itemLink, bag, slot, inv = self:FindItem( name );
		itemLink = self:FindLink(itemLink)
		if ( not itemLink ) then
			itemLink = self:CheckItemDBs( name );
		end
		if ( itemLink and itemLink ~= "?" and itemLink ~= "1" ) then
			local slot = mod( id, 12 );
			local bar = (id-slot) / 12;
			if ( self.db.profile.sets[set] ) then
				local action = self.db.profile.sets[set][bar][slot];
				local iname, ilink = self:ParseActionInfo(action, 1, 4);
				if ( iname ) then
					if ( iname == name and ilink and ilink == "?" ) then
						self:Debug( "Updating item in action #"..id.." in set "..set.." with itemLink." );
						self.db.profile.sets[set][bar][slot] = self:IncActionInfo(action, itemLink, 4);
					else
						self:Debug( "|wAttempted to update itemLink in action #"..id.." but names are wrong?" );
					end
				end
			end
		end
	end
	return itemLink, bag, slot, inv;
end

function SimpleActionSets:SwapSet( set, player )
	if ( not player ) then
		self:Debug( "self:SwapSet - No player name specified, using "..PlrName );
		player = PlrName;
	end

	local actions;
	if ( set ) then
		self:Debug(" self:SwapSet - about to do copy table on self.db.profiles["..player.."].sets["..set.."]" );
		actions = self:CopyTable( self.db.profiles[player].sets[set] );
	elseif ( SASMain:IsVisible() ) then
		self:Debug(" self:SwapSet - about to do copy table on SAS_Temp" );
		actions = self:CopyTable( SAS_Temp );
	else
		return;
	end

	if ( not actions ) then
		self:Debug("|wNo actions to change to.");
		return;
	end

	if ( not inCombat ) then
		self:DoSwap(actions, set, player);
	end
end

function SimpleActionSets:DoSwap(actions, set, player)
	-- Swaps a saved set to the player's real action bars.
	SAS_SwappingSet = true;
	if ( not player or player == PlrName ) then
		self:SetCurrentSet( set );
	end
	ClearCursor();
	for i = 0, 9 do
		if ( self:BarEnabled(i,actions) and not ( not self:BarHasActions(i, actions) and not self.db.profile.emptyBars ) ) then
			self:Debug("Bar "..(i+1).." is enabled");
			for j = 1, 12 do
				local id = j + i*12;
				local msg = "Action "..id;
				if ( actions[i][j]  ) then
					if ( actions[i][j] == 0 ) then
						msg = msg.." forced cleared.";
						self:ClearSlot(id);
					elseif ( actions[i][j] == 1 ) then
						msg = msg.." forced no change.";
					else
						local ename, etexture, erank, elink, emacro = self:GetActionInfo(id);
						local name, texture, rank, link, macro = self:ParseActionInfo( actions[i][j] ); --actions[i][j][1], actions[i][j][2], actions[i][j][3], actions[i][j][4], actions[i][j][5];
						if ( (name == ename and (( rank == erank ) or ( macro and emacro ) or ( link and elink ))) ) then
							msg = msg.." is the same as the one trying to be swapped.";
						else
							if ( macro ) then
								msg = msg.." is a macro.";
								local macroID = self:FindMacro( name, texture, macro )
								if ( macroID ) then
									PickupMacro( macroID );
									PlaceAction(id);
								else
									msg = msg.." |wCouldn't find macro.";
									self:ClearSlot(id);
								end
							elseif ( link and rank~="summonpet" and rank~="flyout") then
								msg = msg.." is an item.";
								if ( not self:PlaceItem( id, link, name, set ) ) then
									msg = msg.." |wCouldn't find item.";
									self:MissingItem( id, {name, texture, link, set} );
								end
							elseif ( rank == "MOUNT" ) then
								msg = msg.." is a companion.";
								local companionId = self:FindCompanion( name, rank );
								if ( companionId ) then
									PickupCompanion( rank, companionId );
									PlaceAction(id);
								else
									msg = msg.." |wCouldn't find companion.";
									self:ClearSlot(id);
								end
							elseif ( rank == "summonpet" ) then
								--name is actually battle pet id as its a unique idenifier
								msg = msg.." is a battle pet.";
								local battlePetName = C_PetJournal.GetPetInfoByPetID(link);
								if (battlePetName) then
									C_PetJournal.PickupPet(link);
									PlaceAction(id);
								else
									msg = msg.." |wCouldn't find battlepet."
									self:ClearSlot(id);
								end
							elseif ( rank == "equipmentset" ) then
								msg = msg.." is an equipment set.";
								local equipSetIcon = GetEquipmentSetInfoByName(name);
								if (equipSetIcon) then
									PickupEquipmentSetByName(name);
									PlaceAction(id);
								else
									msg = msg.." |wCouldn't find equipment set.";
									self:ClearSlot(id);
								end
							elseif ( rank == "flyout") then
								msg = msg.." is a flyout menu.";
								local _, _, _, isKnown = GetFlyoutInfo(link);
								if (isKnown) then
									self:PickupFlyout(link);
									PlaceAction(id);
								else
									msg = msg.." |wCouldn't find flyout menu.";
									self:ClearSlot(id);
								end
							elseif ( name ) then
								msg = msg.." is a spell.";
								local spellNum = self:FindSpell( name, rank );
								if ( spellNum ) then
									PickupSpellBookItem( spellNum, BOOKTYPE_SPELL );
									PlaceAction(id);
								else
									msg = msg.." |wCoulnd't find spell.";
									self:ClearSlot(id);
								end
							else
								msg = msg.." is has no name, link, macro?";
							end
						end

						ClearCursor();
					end
				elseif ( not self.db.profile.noEmptyButtons ) then
					msg = msg.." is empty.";
					self:ClearSlot(id);
				end
				self:Debug(msg);
			end
		elseif ( not self:BarEnabled(i,actions) ) then
			self:Debug("Bar "..(i+1).." not enabled, skipping");
		else
			self:Debug("Bar "..(i+1).." has no actions, skipping");
		end
	end
	ClearCursor();

	SAS_SwappingSet = nil;
end

function SimpleActionSets:CopyTable(copyTable)
	-- properly copies a table instead of referencing the same table, thanks Sallust.
	if ( not copyTable ) then return; end
	local returnTable = {};
	for k, v in pairs(copyTable) do
		if type(v) == "table" then
			returnTable[k] = self:CopyTable(v);
		else
			returnTable[k] = v;
		end
	end
	return returnTable;
end

function SimpleActionSets:SetExists( set )
	-- Check to see if the set already exists
	if self.db.profile.sets[set] then
		return true;
	end
end

function SimpleActionSets:SaveSet(set)
	-- Save a set
	if ( not set ) then
		if ( self:GetCurrentSet() ) then
			set = self:GetCurrentSet();
		else
			return;
		end
	else
		self:SetCurrentSet(set);
	end
	local actions;
	if ( SASMain:IsVisible() and SAS_Temp ) then
		actions = self:CopyTable(SAS_Temp);
	else
		actions = self:IterateActions();
	end
	self:Print(L["SAS_TEXT_SAVING"]..set);
	self.db.profile.sets[set] = self:CopyTable(actions);
	self:Actions_Load( set );
end

function SimpleActionSets:Delete( set )
	-- Delete a set
	self:Print(L["SAS_TEXT_DELETING"]..set);
	self.db.profile.sets[set] = nil;
	if ( self:GetCurrentSet() == set ) then
		self:SetCurrentSet();
	end
	self:Actions_Load();
end

function SimpleActionSets:SetCurrentSet( set )
	self:Debug("Changing current set to " .. (set or "nil"));
	self.db.profile.currentSet = set;
end

function SimpleActionSets:GetCurrentSet()
	if (self.db) then
		return self.db.profile.currentSet;
	else
		return nil
	end
end

function SimpleActionSets:RestoreBackUp()
	SimpleActionSets:SwapSet( PlrName, "BackUp" );
end

function SimpleActionSets:Console( msg )
	if ( strlower(strsub( msg, 1, 4 )) == "swap" ) then
		local set = strsub( msg, 6 );
		if ( set and self:SetExists( set ) ) then
			self:SwapSet( set );
		else
			self:Print( gsub(L["SAS_TEXT_CONSOLE_NOVALID"], "%%s", set) );
		end
	elseif ( strlower(strsub( msg, 1, 4 )) == "save" ) then
		local set = strsub( msg, 6 );
		if ( set ) then
			if ( self:SetExists( set ) ) then
			else
				self:SaveSet( set );
			end
		end
	elseif ( strlower(strsub( msg, 1, 5 )) == "debug" ) then
		self.db["debug"] = not self.db["debug"];
	elseif ( strlower(strsub( msg, 1, 4 )) == "show" or msg == "" ) then
		self:Main_Toggle();
	else
		self:Print( L["SAS_TITLE"]..L["SAS_TEXT_CONSOLE_HELP"] );
	end
end

function SimpleActionSets:tLink( itemLink )
	-- truncate item links to item ids
	if ( not itemLink ) then return; end
	for num in gmatch( itemLink, "m:(%d+)" ) do return num; end
	return itemLink;
end

function SimpleActionSets:actionComp( a, b )
	if a and b then
		local A = { self:ParseActionInfo( a ) };
		local B = { self:ParseActionInfo( b ) };
		for k, v in pairs(A) do
			if ( k ~= 4 and (not B[k] or v ~= B[k]) ) then
				self:Debug( k.." "..v.." does not == "..B[k] );
				return 1;
			end
		end
	else
		if ( a ) then
			self:Debug( "no b" );
		elseif ( b ) then
			self:Debug( "no a" );
		else
			self:Debug( "no a or b" );
		end
		return 1;
	end
end

function SimpleActionSets:CompareSet( set1, set2, strict )
	-- Compare two sets to see if they're similar
	if ( not set1 or not set2 ) then
		if ( not set1 ) then
			self:Debug( "no set1" );
		elseif ( not set2 ) then
			self:Debug( "no set2" );
		else
			self:Debug( "no set1 or set2" );
		end
		return 1;
	end

	for i=0, 9 do
		if ( set1[i] ) then
			if ( strict == 2 ) then
				if ( set1[i][0] ~= set2[i][0] ) then
					return 1;
				end
			end
			for j=1, 12 do
				if ( set1[i][j] ) then
					if ( self:actionComp( set1[i][j], set2[i][j] ) ) then
						self:Debug( "returning at "..i.." "..j );
						return 1;
					end
				elseif ( strict and set2[i][j] ) then
					self:Debug( "strict returning at "..i.." "..j.." no set1 but set2" );
					return 1;
				end
			end
		elseif ( strict and set2[i] ) then
			self:Debug( "strict returning at "..i.." no set1 but set2" );
			return 1;
		end
	end
end

function SimpleActionSets:SetReturnSlot(slot,blah)
	self:Debug(slot.." "..blah);
	SAS_ReturnAction = slot;
	if ( self:GetMissingItemInfo( slot ) ) then
		self.db.profile["MissingItems"][slot] = nil;
	end
end
hooksecurefunc("PickupAction", function(slot) SimpleActionSets:SetReturnSlot(slot,1) end);
hooksecurefunc("PlaceAction", function(slot) SimpleActionSets:SetReturnSlot(slot,2) end);
hooksecurefunc("UseAction", function(slot) SimpleActionSets:SetReturnSlot(slot,3) end);


---------------------------------------------
-- Hook functions to track picked up items --
---------------------------------------------

-- All hooks replaced with new function, yay for GetCursorInfo()!
function SimpleActionSets:BuildCursorAction()
	self:FakeDrag_Drop(1);
	local curtype, detail, subdetail = GetCursorInfo();
	if ( not curtype ) then
		return;
	elseif ( curtype == "item" ) then
		local name,_,_,_,_,_,_,_,_,texture = GetItemInfo(detail);
		if ( name ) then
			return self:BuildActionInfo( name, texture, nil, detail );
		end
	elseif ( curtype == "spell" and detail > 0 ) then
		local texture = GetSpellTexture( detail, BOOKTYPE_SPELL );
		if ( texture ) then
			local name, rank = GetSpellBookItemName( detail, BOOKTYPE_SPELL );
			return self:BuildActionInfo( name, texture, rank );
		end
	elseif ( curtype == "companion" ) then
		local _,name,_,texture = GetCompanionInfo(subdetail, detail);
		if ( name ) then
			return self:BuildActionInfo( name, texture, subdetail );
		end
	elseif ( curtype == "battlepet" ) then
		local _, _, _, _, _, _, name, texture = C_PetJournal.GetPetInfoByPetID( detail );
		return self:BuildActionInfo( name, texture, "summonpet", detail);
	elseif ( curtype == "equipmentset" ) then
		local icon = GetEquipmentSetInfoByName(detail);
		return self:BuildActionInfo( detail, "Interface\\Icons\\"..icon, curtype);
	elseif ( curtype == "flyout" ) then
		local name = GetFlyoutInfo(detail);
		return self:BuildActionInfo( name, subdetail, curtype, detail );
	elseif ( curtype == "macro" ) then
		local name, texture = GetMacroInfo(detail);
		if ( name ) then
			return self:BuildActionInfo( name, texture, nil, nil, detail );
		end
	end
end


--------------------------------------------
-- Hook functions to drop fake drag frame --
--------------------------------------------

function SimpleActionSets:forceFakeDrop()
	self:FakeDrag_Drop(1);
end
hooksecurefunc("CameraOrSelectOrMoveStart", function() SimpleActionSets:forceFakeDrop() end);
hooksecurefunc("TurnOrActionStart", function() SimpleActionSets:forceFakeDrop() end);